@@ -16,7 +16,251 @@ from scipy.interpolate import interp2d
 from matplotlib.widgets import Slider, Button, TextBox
 from matplotlib.artist import ArtistInspector
 import PhyREC.SignalProcess as Spro
-from . import SpectColBars
+from PhyREC import SpectColBars
+from sklearn.impute import KNNImputer
+from copy import deepcopy
+
+
+class ImageSequence(pq.Quantity):
+   
+    def __new__(cls, signal, sampling_rate,  t_start=0*pq.s, units=None, dtype=None, copy=True,
+                name=None, **kwargs):
+       
+        signal = cls._rescale(cls, signal=signal, units=units)
+        obj = pq.Quantity(signal, units=units, dtype=dtype, copy=copy).view(cls)                                         
+        
+        obj.sampling_rate = sampling_rate
+        obj.t_start = t_start
+        obj.name = name
+        obj.annotations = {}
+        return obj
+    
+    def __array_finalize__(self, obj):
+        if obj is None:
+            return
+        super(ImageSequence, self).__array_finalize__(obj)
+        self.sampling_rate = getattr(obj, 'sampling_rate', None)
+        self.t_start = getattr(obj, 't_start', None)
+        self.name = getattr(obj, 'name', None)
+        self.annotations = getattr(obj, 'annotations', {})
+
+    
+    def __repr__(self):
+        return("{cls} {frame} frames with {width} pixels of width and {height} pixels of height \n"
+                "units {units} \n"
+                "sampling_rate {sampling_rate} \n"
+                "Duration {t_start} --  {t_stop}\n"
+                "datatype {dtype} \n"
+                "name {name} ".format(cls=self.__class__.__name__,
+                                            frame=self.shape[0],
+                                            height=self.shape[1],
+                                            width=self.shape[2],
+                                            units=self.units,
+                                            dtype=self.dtype,
+                                            sampling_rate=self.sampling_rate,
+                                            t_start=self.t_start,
+                                            t_stop=self.t_stop,
+                                            name=self.name))
+
+    def __deepcopy__(self, memo):
+        """
+            Create a deep copy of the data object.
+            All attributes and annotations are also deep copied.
+            References to parent objects are not kept, they are set to None.
+
+
+            :param memo: (dict) Objects that have been deep copied already
+            :return: (DataObject) Deep copy of the input DataObject
+        """
+        cls = self.__class__
+        necessary_attrs = {'signal':self,
+                           'units':self.units,
+                           'sampling_rate': self.sampling_rate,
+                           'name': self.name,
+                           't_start': self.t_start}
+        # Create object using constructor with necessary attributes
+        new_obj = cls(**necessary_attrs)
+        # Add all attributes
+
+        return new_obj
+
+    def _rescale(self, signal, units=None):
+        '''
+        Check that units are present, and rescale the signal if necessary.
+        This is called whenever a new signal is
+        created from the constructor. See :meth:`__new__' in
+        :class:`AnalogSignal` and :class:`IrregularlySampledSignal`
+        '''
+        if units is None:
+            if not hasattr(signal, "units"):
+                raise ValueError("Units must be specified")
+        elif isinstance(signal, pq.Quantity):
+            # This test always returns True, i.e. rescaling is always executed if one of the units
+            # is a pq.CompoundUnit. This is fine because rescaling is correct anyway.
+            if pq.quantity.validate_dimensionality(units) != signal.dimensionality:
+                signal = signal.rescale(units)
+        return signal
+
+    def rescale(self, units):
+        obj = super(ImageSequence, self).rescale(units)
+
+        obj.sampling_rate = self.sampling_rate
+        obj.t_start = self.t_start
+        obj.name = self.name
+        obj.annotations = self.annotations        
+        return obj.view(ImageSequence)
+    
+    def time_index(self, t):
+        """Return the array index corresponding to the time `t`"""
+        i = (t - self.t_start) * self.sampling_rate
+        i = int(np.rint(i.simplified.magnitude))
+        return i
+
+    def time_slice(self, t_start, t_stop):
+        '''
+        Creates a new AnalogSignal corresponding to the time slice of the
+        original AnalogSignal between times t_start, t_stop. Note, that for
+        numerical stability reasons if t_start does not fall exactly on
+        the time bins defined by the sampling_period it will be rounded to
+        the nearest sampling bin. The time bin for t_stop will be chosen to
+        make the duration of the resultant signal as close as possible to
+        t_stop - t_start. This means that for a given duration, the size
+        of the slice will always be the same.
+        '''
+
+        # checking start time and transforming to start index
+        if t_start is None:
+            i = 0
+            t_start = 0 * pq.s
+        else:
+            i = self.time_index(t_start)
+
+        # checking stop time and transforming to stop index
+        if t_stop is None:
+            j = len(self)
+        else:
+            delta = (t_stop - t_start) * self.sampling_rate
+            j = i + int(np.rint(delta.simplified.magnitude))
+
+        if (i < 0) or (j > len(self)):
+            raise ValueError('t_start, t_stop have to be within the analog \
+                              signal duration')
+
+        # Time slicing should create a deep copy of the object
+        obj = deepcopy(self[i:j, :, :])
+
+        obj.t_start = self.t_start + i * self.sampling_period      
+
+        return obj
+
+    @property
+    def duration(self):
+        '''
+        Signal duration
+
+        (:attr:`size` * :attr:`sampling_period`)
+        '''
+        return self.shape[0] / self.sampling_rate
+
+    @property
+    def t_stop(self):
+        '''
+        Time when signal ends.
+
+        (:attr:`t_start` + :attr:`duration`)
+        '''
+        return self.t_start + self.duration
+
+    @property
+    def times(self):
+        '''
+        The time points of each sample of the signal
+
+        (:attr:`t_start` + arange(:attr:`shape`)/:attr:`sampling_rate`)
+        '''
+        return self.t_start + np.arange(self.shape[0]) / self.sampling_rate
+   
+    @property
+    def sampling_period(self):
+        '''
+        Interval between two samples.
+
+        (1/:attr:`sampling_rate`)
+        '''
+        return 1. / self.sampling_rate
+    
+    def annotate(self, **annotations):
+        """
+        Add annotations (non-standardized metadata) to a Neo object.
+
+        Example:
+
+        >>> obj.annotate(key1=value0, key2=value1)
+        >>> obj.key2
+        value2
+        """        
+        self.annotations.update(annotations)
+    
+    def __array_ufunc__(self, ufunc, method, *inputs, out=None, **kwargs):
+        args = []
+        in_no = []
+        for i, input_ in enumerate(inputs):
+            if isinstance(input_, ImageSequence):
+                in_no.append(i)
+                args.append(input_.view(pq.Quantity))
+            else:
+                args.append(input_)
+
+        results = super(ImageSequence, self).__array_ufunc__(ufunc, method, *args, **kwargs)
+        if results is NotImplemented:
+            return NotImplemented
+
+        results = results.view(ImageSequence)
+        results.sampling_rate = inputs[0].sampling_rate
+        results.t_start = inputs[0].t_start
+        results.name = inputs[0].name
+        results.annotations = inputs[0].annotations
+        
+        return results
+        
+        # if method == 'at':
+        #     if isinstance(inputs[0], A):
+        #         inputs[0].info = info
+        #     return
+
+        # if ufunc.nout == 1:
+        #     results = (results,)
+
+        # results = tuple((np.asarray(result).view(A)
+        #                  if output is None else output)
+        #                 for result, output in zip(results, outputs))
+        # if results and isinstance(results[0], A):
+        #     results[0].info = info
+
+        # return results[0] if len(results) == 1 else results
+    def duplicate_with_new_data(self, signal, units=None):
+        '''
+        Create a new signal with the same metadata but different data.
+        Required attributes of the signal are used.
+        Note: Array annotations can not be copied here because length of data can change
+        '''
+        if units is None:
+            units = self.units
+        # else:
+        #     units = pq.quantity.validate_dimensionality(units)
+
+        # signal is the new signal
+        necessary_attrs = {'signal': signal,
+                           'units':self.units,
+                           'sampling_rate': self.sampling_rate,
+                           'name': self.name,
+                           't_start': self.t_start}
+        new = self.__class__(**necessary_attrs)
+        new.annotations.update(self.annotations)
+        # Note: Array annotations are not copied here, because it is not ensured
+        # that the same number of signals is used and they would possibly make no sense
+        # when combined with another signal
+        return new
 
 #from NeoInterface import NeoTrain
 
@@ -368,6 +612,8 @@ class WaveSlot():
             self.name = self.LineKwargs['label']
 
         self.TrialLineKwargs['color'] = self.LineKwargs['color']
+        
+        self.current_time = None
 
     def CheckTime(self, Time):
         if Time is None:
@@ -420,8 +666,10 @@ class WaveSlot():
                                   sig,
                                   **self.LineKwargs
                                   )
-        self.Ax.set_xlim(left=sig.t_start.rescale('s').magnitude,
-                         right=sig.t_stop.rescale('s').magnitude)
+        self.current_time = (sig.t_start.rescale('s'),
+                             sig.t_stop.rescale('s'))
+        self.Ax.set_xlim(left=self.current_time[0],
+                         right=self.current_time[1])
         self.Line = self.Lines[0]
 
     def CalcAvarage(self, TimeAvg, TimesEvent, Units=None,
@@ -468,10 +716,22 @@ class ControlFigure():
 
         self.pltSL = pltSL
 
-        TMax = np.max([sl.Signal.t_stop.rescale('s') for sl in pltSL.Slots])
-        TMin = np.min([sl.Signal.t_start.rescale('s') for sl in pltSL.Slots])
+        self.MapSlots = []
+        for sl in pltSL.Slots:
+            if hasattr(sl, 'Map'):
+                self.MapSlots.append(sl)
+        
+        TMax = []
+        TMin = []
+        for sl in pltSL.Slots:
+            if not hasattr(sl, 'Map'):
+                TMax.append(sl.Signal.t_stop.rescale('s'))
+                TMin.append(sl.Signal.t_start.rescale('s'))
+                
+        TMax = np.max(TMax)
+        TMin = np.min(TMin)
 
-        self.Fig, ax = plt.subplots(6, 1, figsize=figsize)
+        self.Fig, ax = plt.subplots(10, 1, figsize=figsize)
         self.sTstart = Slider(ax[0],
                               label='TStart [s]',
                               valmax=TMax,
@@ -501,14 +761,71 @@ class ControlFigure():
         self.OldStart = 0
         self.OldStop = 0
         
-        self.bStart = Button(ax[4],
-                             label='Start')        
-        self.bStart.on_clicked(self.StartAnimation)
-        self.Timer = None
-        self.TextInterval = TextBox(ax[5],
-                                 'Interval [ms]',
-                                 initial='2000')
+        # self.bStart = Button(ax[4],
+        #                      label='Start')        
+        # self.bStart.on_clicked(self.StartAnimation)
+        # self.Timer = None
+        # self.TextInterval = TextBox(ax[5],
+        #                             'Interval [ms]',
+        #                             initial='2000')
 
+        self.bStartMapAni = Button(ax[6],
+                                   label='Annimate Maps START')    
+        self.TextMapAniSpeed = TextBox(ax[7],
+                                       'Map Speed [1x]',
+                                       initial='1')   
+        self.TextMapAniPoints = TextBox(ax[8],
+                                        'Map Points [n]',
+                                        initial='500') 
+        self.bStartMapAni.on_clicked(self.StartMapAnimation)
+        self.TimerMap = None
+
+        self.bStartSetZero = Button(ax[9],
+                                   label='Set Zero at Start Time')
+        self.bStartSetZero.on_clicked(self.BtSetZero)
+    
+    def BtSetZero(self, val):
+        Twind = (self.sTstart.val * pq.s,
+                 (self.sTstart.val+5) * pq.s)        
+           
+        for sl in self.pltSL.Slots:
+            if 'LiveZero' in sl.Signal.annotations:
+                if sl.Signal.annotations['LiveZero']:
+                    # print(sl.Signal.name)
+                    sl.Signal = Spro.SetZero(sl.Signal, TWind=Twind)
+        self.Update(None)
+
+    def StartMapAnimation(self, val):
+        if self.TimerMap is not None:
+            self.bStartMapAni.label.set_label('Annimate Maps START')
+            self.TimerMap.stop()
+            self.TimerMap = None
+            return
+
+        twind = (self.sTstart.val * pq.s,
+                 self.sTstart.val * pq.s + self.sTshow.val * pq.s)
+
+        points = int(self.TextMapAniPoints.text)
+        speed = float(self.TextMapAniSpeed.text)
+        interval = ((twind[1] - twind[0])/points)/speed
+        self.MapCount = 0
+        self.MapTimes = np.linspace(twind[0], twind[1], points)
+        print('Start', interval)
+        
+        self.TimerMap = self.Fig.canvas.new_timer(interval=interval.rescale('ms'))
+        self.TimerMap.add_callback(self.UpdateMapAnimation)
+        self.TimerMap.start()
+        self.bStartMapAni.label.set_label('Annimate Maps STOP')
+
+    def UpdateMapAnimation(self):
+        for sl in self.MapSlots:
+            sl.PlotSignal((self.MapTimes[self.MapCount],
+                           self.MapTimes[self.MapCount]+1*pq.s))
+
+        if self.MapCount >= (len(self.MapTimes)-1):
+            self.MapCount = 0
+        else:
+            self.MapCount += 1
 
     def StartAnimation(self, val):         
         if self.Timer is not None:
@@ -579,9 +896,9 @@ class ControlFigure():
 
     def SetTimes(self, twind):
         self.Refresh = False
-        self.TextStop.set_val(str(twind[1]))
-        self.TextStart.set_val(str(twind[0]))
-        self.TextStop.set_val(str(twind[1]))
+        self.TextStop.set_val(str(np.array(twind[1])))
+        self.TextStart.set_val(str(np.array(twind[0])))
+        self.TextStop.set_val(str(np.array(twind[1])))
         self.Refresh = True
 
 
@@ -732,10 +1049,11 @@ class PlotSlots():
 #                sl.Ax.set_xlim(left=Time[0].magnitude)
 #            if Time[1] is not None:
 #                sl.Ax.set_xlim(right=Time[1].magnitude)
+            if sl.current_time is not None:
+                if not hasattr(sl, 'Map'):
+                    self.current_time = sl.current_time
 
-        self.current_time = sl.Ax.get_xlim()
-
-        if self.CtrFig is not None:
+        if self.CtrFig is not None:            
             self.CtrFig.SetTimes(self.current_time)
 
     def PlotEvents(self, Times, Labels=None, lAx=0, fontsize='xx-small',
@@ -796,10 +1114,77 @@ class PlotSlots():
         self.FormatFigure()
         return MeanSigs
     
+    
+class ImgSlot():
+    DefAxKwargs = {}
+    DefImKwargs = {
+                    'vmin': -10,
+                    'vmax': 10,
+                    'cmap': 'seismic',
+                    'interpolation': 'bicubic',
+                    }
+    
+    def __init__(self, Signal, Ax=None,
+                 AxKwargs=None, Units=None, imKwargs=None,):
+        
+        self.Signal = Signal
+        self.Ax = Ax
+        
+        self.Map=True
 
+        self.current_time = None
+        self.units = Units
+        
+        self.imKwargs = self.DefImKwargs.copy()
+        if imKwargs is not None:
+            self.imKwargs.update(imKwargs)
+
+        self.Img = self.Ax.imshow(np.array(self.Signal[0,:,:]),
+                                  **self.imKwargs,
+                                  )
+
+    def CheckTime(self, Time):
+        if Time is None:
+            return (self.Signal.t_start,
+                    self.Signal.t_start + self.Signal.sampling_period)
+
+        if len(Time) == 1:
+            return (Time[0], Time[0] + self.Signal.sampling_period)
+
+        if Time[0] is None or Time[0] < self.Signal.t_start:
+            Tstart = self.Signal.t_start
+        else:
+            Tstart = Time[0]
+
+        if Time[0] > self.Signal.t_stop:
+            Tstart = self.Signal.t_stop - 2*self.Signal.sampling_period
+
+        return (Tstart, Tstart+ self.Signal.sampling_period)
+
+    def GetSignal(self, Time, Units=None):
+        if Units is None:
+            _Units = self.units
+        else:
+            _Units = Units
+        Time = self.CheckTime(Time)
+        sig = self.Signal.time_slice(Time[0], Time[1])
+        if _Units is not None:
+            sig = sig.rescale(_Units)
+        self.units = sig.units
+        return sig
+
+        
+    def PlotSignal(self, Time, Units=None):
+        sig = self.GetSignal(Time, Units)
     
-    
-    
-    
+        self.Img.set_array(np.array(sig[0, :, :]))        
+        self.Ax.set_title('{0:.1f}'.format(sig.t_start))
+        self.current_time = (sig.t_start.rescale('s'),
+                             sig.t_stop.rescale('s'))
+        self.Ax.figure.canvas.draw()
+
+    def UpdateAxKwargs(self, AxKwargs):
+        pass
+  
     
     
\ No newline at end of file
