@@ -8,12 +8,10 @@ Created on Wed Apr 11 10:27:23 2018
 import numpy as np
 from scipy import signal
 from fractions import Fraction
-from PhyREC.NeoInterface import NeoSegment, NeoSignal, NeoTrain
+from neo.core import AnalogSignal, SpikeTrain
 import PhyREC.SignalAnalysis as Ran
 import quantities as pq
-import matplotlib.pyplot as plt
 import matplotlib.mlab as mlab
-import elephant
 import scipy.stats as stats
 from scipy import interpolate
 import sys
@@ -67,7 +65,7 @@ def Spectrogram(sig, Fres=2*pq.Hz, TimeRes=0.01*pq.s,
             return s
         else:
             NormSig = s.time_slice(NormTime[0], NormTime[1])
-            mean = np.mean(NormSig, axis=0)           
+            mean = np.mean(NormSig, axis=0)
             return (s / mean)
 
 
@@ -84,14 +82,14 @@ def AvgSpectrogram(sig, TimesEvent, TimeAvg, SpecArgs,
         elif et+TimeAvg[1] > sig.t_stop:
             print(et, ' not valid')
             continue
-        
+
         Trials += 1
         s = sig.time_slice(et+TimeAvg[0], et+TimeAvg[1])
         if TrialProcessChain is not None:
             st = ApplyProcessChain(s, TrialProcessChain)
         else:
-            st = s  
-      
+            st = s
+
         spect = Spectrogram(st, **SpecArgs)
         Acc = Acc + np.array(spect) if Acc.size else np.array(spect)
 
@@ -113,12 +111,12 @@ def AvgSpectrogram(sig, TimesEvent, TimeAvg, SpecArgs,
     NormSig = AvgSpect.time_slice(NormTime[0], NormTime[1])
     mean = np.mean(NormSig, axis=0)
     std = np.std(NormSig, axis=0)
-    
+
     if AvgSpectNorm == 'Zscore':
         AvgNormSpect = (AvgSpect - mean) / std
     else:
         AvgNormSpect = AvgSpect / mean
-    
+
     return AvgNormSpect
 
 
@@ -133,15 +131,15 @@ def TrigAveraging(sig, TimesEvent, TimeAvg, TrialProcessChain=None):
         elif et+TimeAvg[1] > sig.t_stop:
             print(et, ' not valid')
             continue
-    
+
         Samp1 = sig.time_index(et+TimeAvg[0])
-        s = sig[Samp1:Samp1+nSamps, :]    
-    
+        s = sig[Samp1:Samp1+nSamps, :]
+
         if TrialProcessChain is not None:
             st = ApplyProcessChain(s, TrialProcessChain)
         else:
             st = s
-    
+
         st.t_start = TimeAvg[0]
         st.array_annotations = {}
         st.annotations = {}
@@ -156,12 +154,13 @@ def TrigAveraging(sig, TimesEvent, TimeAvg, TrialProcessChain=None):
     avg.annotate(acc=acc)
     return avg
 
+
 def Derivative(sig):
-    derivative_sig = NeoSignal(
-            np.diff(sig.as_quantity(), axis=0) / sig.sampling_period,
-            t_start=sig.t_start+sig.sampling_period/2,
-            sampling_period=sig.sampling_period,
-            name=sig.name)
+    derivative_sig = AnalogSignal(
+        np.diff(sig.as_quantity(), axis=0) / sig.sampling_period,
+        t_start=sig.t_start+sig.sampling_period/2,
+        sampling_period=sig.sampling_period,
+        name=sig.name)
 
     return derivative_sig
 
@@ -186,10 +185,10 @@ def RemoveDC(sig, Type='constant'):
 
 def SetZero(sig, TWind=None):
     if TWind is None:
-        TWind=(sig.t_start, sig.t_start+30*pq.s)
+        TWind = (sig.t_start, sig.t_start+30*pq.s)
     st = np.array(sig)
     # offset = np.mean(sig.GetSignal(TWind))
-    offset = np.mean(sig.time_slice(TWind[0], TWind[1]))
+    offset = np.mean(sig.time_slice(TWind[0], TWind[1]), axis=0)
     print(sig.name, offset)
     st_corrected = st-offset.magnitude
     return sig.duplicate_with_new_data(signal=st_corrected*sig.units)
@@ -241,13 +240,13 @@ def Filter(sig, Type, Order, Freqs):
 
     # b, a = signal.butter(Order, freqs, Type)
     # st = signal.filtfilt(b, a, st, axis=0)
-    
+
     sos = signal.butter(Order, freqs, Type, output='sos')
     st = signal.sosfiltfilt(sos, st, axis=0)
-    
+
     # DbgFplt.PlotResponse(a, b, Fs)
     DbgFplt.PlotResponse(sos, Fs)
-    
+
     return sig.duplicate_with_new_data(signal=st*sig.units)
 
 
@@ -259,8 +258,7 @@ def power_sliding(x, axis=None):
     return np.mean(x**2, axis=axis)
 
 
-
-def sliding_window(sig, timewidth, func=None, steptime=None,**kwargs):
+def sliding_window(sig, timewidth, func=None, steptime=None, **kwargs):
 
     if steptime is None:
         steptime = timewidth/10
@@ -268,29 +266,32 @@ def sliding_window(sig, timewidth, func=None, steptime=None,**kwargs):
     if func is None:
         func = rms
 
-    window_size = int(timewidth.rescale('s') / sig.sampling_period.rescale('s'))
+    window_size = int(timewidth.rescale(
+        's') / sig.sampling_period.rescale('s'))
     timewidth = sig.sampling_period.rescale('s')*window_size
     step_size = int(steptime.rescale('s') / sig.sampling_period.rescale('s'))
     steptime = sig.sampling_period.rescale('s')*step_size
 
     axis = 0
     shape = list(sig.shape)
-    shape[axis] = np.floor(sig.shape[axis] / step_size - window_size / step_size + 1).astype(int)
+    shape[axis] = np.floor(sig.shape[axis] / step_size -
+                           window_size / step_size + 1).astype(int)
     shape.append(window_size)
 
     strides = list(sig.strides)
     strides[axis] *= step_size
     strides.append(sig.strides[axis])
 
-    strided = np.lib.stride_tricks.as_strided(sig, shape=shape, strides=strides)
+    strided = np.lib.stride_tricks.as_strided(
+        sig, shape=shape, strides=strides)
 
-    st = func(strided, axis=-1,**kwargs)
- 
-    return NeoSignal(signal=st,
-                     units=sig.units,
-                     t_start=sig.t_start + timewidth/2,
-                     name=sig.name,
-                     sampling_rate=1/steptime)
+    st = func(strided, axis=-1, **kwargs)
+
+    return AnalogSignal(signal=st,
+                        units=sig.units,
+                        t_start=sig.t_start + timewidth/2,
+                        name=sig.name,
+                        sampling_rate=1/steptime)
 
 
 def ThresholdTrianGen(sig, RelaxTime=0.4*pq.s, threshold=None):
@@ -302,111 +303,116 @@ def ThresholdTrianGen(sig, RelaxTime=0.4*pq.s, threshold=None):
                                        RelaxTime=RelaxTime)
     inttimes = np.array(inttimes)
 
-    return NeoTrain(times=inttimes,
-                    units='s',
-                    t_start=sig.t_start,
-                    t_stop=sig.t_stop,
-                     )
+    return SpikeTrain(times=inttimes,
+                      units='s',
+                      t_start=sig.t_start,
+                      t_stop=sig.t_stop,
+                      )
 
 
-def ThresholdInstantRate(sig, RelaxTime=0.1*pq.s, threshold=None,
-                         OutSampling=0.01*pq.s,):
-
-    
-
-    return elephant.statistics.instantaneous_rate(ThresholdTrianGen(sig,
-                                                                    RelaxTime,
-                                                                    threshold),
-                                                  sampling_period=OutSampling)
+# def ThresholdInstantRate(sig, RelaxTime=0.1*pq.s, threshold=None,
+#                          OutSampling=0.01*pq.s,):
 
 
-def HilbertInstantFreq(sig, MaxFreq=20, MinFreq=0):
-    SigH = elephant.signal_processing.hilbert(sig)
-    insfreq = np.diff(np.angle(SigH)[:, 0]) / np.diff(SigH.times)
-
-    return NeoSignal(signal=np.clip(insfreq.magnitude, 0, 20),
-                     units='Hz',
-                     name=sig.name,
-                     sampling_rate=SigH.sampling_rate,
-                     t_start=SigH.t_start)
-
-def HilbertAngle(sig):
-    SigH = elephant.signal_processing.hilbert(sig)   
-    return sig.duplicate_with_new_data(signal=np.angle(SigH),
-                                       units=pq.radians)
+#     return elephant.statistics.instantaneous_rate(ThresholdTrianGen(sig,
+#                                                                     RelaxTime,
+#                                                                     threshold),
+#                                                   sampling_period=OutSampling)
 
 
-def HilbertAmp(sig):
-    SigH = elephant.signal_processing.hilbert(sig)
-    
-    return sig.duplicate_with_new_data(signal=np.array(np.abs(SigH))*sig.units)
+# def HilbertInstantFreq(sig, MaxFreq=20, MinFreq=0):
+#     SigH = elephant.signal_processing.hilbert(sig)
+#     insfreq = np.diff(np.angle(SigH)[:, 0]) / np.diff(SigH.times)
+
+#     return AnalogSignal(signal=np.clip(insfreq.magnitude, 0, 20),
+#                      units='Hz',
+#                      name=sig.name,
+#                      sampling_rate=SigH.sampling_rate,
+#                      t_start=SigH.t_start)
+
+# def HilbertAngle(sig):
+#     SigH = elephant.signal_processing.hilbert(sig)
+#     return sig.duplicate_with_new_data(signal=np.angle(SigH),
+#                                        units=pq.radians)
+
+
+# def HilbertAmp(sig):
+#     SigH = elephant.signal_processing.hilbert(sig)
+
+#     return sig.duplicate_with_new_data(signal=np.array(np.abs(SigH))*sig.units)
 
 
 def SplineSmooth(sig, sFact=2, **kwargs):
     s = sig.shape[0]/sFact
-    spl = UnivariateSpline(sig.times, sig, s=s)    
+    spl = UnivariateSpline(sig.times, sig, s=s)
     return sig.duplicate_with_new_data(signal=spl(sig.times)*sig.units)
 
-def MedianFilt(sig, window_size=None, **kwargs): #window_size in pq.s 
-    if window_size==None:
-        kernel_size=int(sig.shape[0]/10)
+
+def MedianFilt(sig, window_size=None, **kwargs):  # window_size in pq.s
+    if window_size == None:
+        kernel_size = int(sig.shape[0]/10)
     else:
-        kernel_size=int(sig.sampling_rate*window_size)
-    if bool(kernel_size & 1): ##kernel_size has to be an odd number
-        kernel_size=kernel_size
+        kernel_size = int(sig.sampling_rate*window_size)
+    if bool(kernel_size & 1):  # kernel_size has to be an odd number
+        kernel_size = kernel_size
     else:
-        kernel_size=kernel_size-1
-    SigFilt=medfilt(np.array(sig).reshape(len(sig),), kernel_size=kernel_size)    # TODO change by axis
+        kernel_size = kernel_size-1
+    SigFilt = medfilt(np.array(sig).reshape(len(sig),),
+                      kernel_size=kernel_size)    # TODO change by axis
     return sig.duplicate_with_new_data(signal=np.array(SigFilt)*sig.units)
 
-   
+
 def Strid_signal(sig, timewidth, steptime=None):
     if steptime is None:
         steptime = timewidth/10
 
-    window_size = int(timewidth.rescale('s') / sig.sampling_period.rescale('s'))
+    window_size = int(timewidth.rescale(
+        's') / sig.sampling_period.rescale('s'))
     timewidth = sig.sampling_period.rescale('s')*window_size
     step_size = int(steptime.rescale('s') / sig.sampling_period.rescale('s'))
     steptime = sig.sampling_period.rescale('s')*step_size
 
     axis = 0
     shape = list(sig.shape)
-    shape[axis] = np.floor(sig.shape[axis] / step_size - window_size / step_size + 1).astype(int)
+    shape[axis] = np.floor(sig.shape[axis] / step_size -
+                           window_size / step_size + 1).astype(int)
     shape.append(window_size)
 
     strides = list(sig.strides)
     strides[axis] *= step_size
     strides.append(sig.strides[axis])
 
-    strided = np.lib.stride_tricks.as_strided(sig, shape=shape, strides=strides)
+    strided = np.lib.stride_tricks.as_strided(
+        sig, shape=shape, strides=strides)
 
     return strided
 
+
 def CrossCorr(x1, x2):
-    max_cross_corr=[]
-    for row in range(x1.shape[0]):   
-#        print(x1.shape)
-#        print (x1[row,:,:].shape)
-#        print(x2.shape)
-#        print (x2[row,:,:].shape)
-#        print(row)
-        res=xcorr(
-                        np.array(x1[row,:,:]).reshape((x1[row,:,:].shape[-1],)),
-                      np.array(x2[row,:,:]).reshape((x2[row,:,:].shape[-1],)),maxlags=None)
-        cross_correlation=max(res[1])
+    max_cross_corr = []
+    for row in range(x1.shape[0]):
+        #        print(x1.shape)
+        #        print (x1[row,:,:].shape)
+        #        print(x2.shape)
+        #        print (x2[row,:,:].shape)
+        #        print(row)
+        res = xcorr(
+            np.array(x1[row, :, :]).reshape((x1[row, :, :].shape[-1],)),
+            np.array(x2[row, :, :]).reshape((x2[row, :, :].shape[-1],)), maxlags=None)
+        cross_correlation = max(res[1])
         max_cross_corr.append(cross_correlation)
-        
+
     return max_cross_corr
 
 
 def PearsonCorr(x1, x2):
-    pearson_corr=[]
-    for row in range(x1.shape[0]):   
-        res=stats.pearsonr(
-                        np.array(x1[row,:,:]).reshape((x1[row,:,:].shape[-1],)),
-                      np.array(x2[row,:,:]).reshape((x2[row,:,:].shape[-1],)))
+    pearson_corr = []
+    for row in range(x1.shape[0]):
+        res = stats.pearsonr(
+            np.array(x1[row, :, :]).reshape((x1[row, :, :].shape[-1],)),
+            np.array(x2[row, :, :]).reshape((x2[row, :, :].shape[-1],)))
         pearson_corr.append(res[0])
-        
+
     return pearson_corr
 
 
@@ -420,101 +426,101 @@ def sliding_window_2sigs(sig1, sig2, timewidth, func=None, steptime=None):
 
     st = func(strided1, strided2)
 
-    return NeoSignal(signal=st,
-                     units=sig1.units,
-                     t_start=sig1.t_start + timewidth/2,
-                     name='CrossCorr'+sig1.name+sig1.name,
-                     sampling_rate=1/steptime)    
-    
-    
+    return AnalogSignal(signal=st,
+                        units=sig1.units,
+                        t_start=sig1.t_start + timewidth/2,
+                        name='CrossCorr'+sig1.name+sig1.name,
+                        sampling_rate=1/steptime)
+
+
 def xcorr(x, y, normed=True, detrend=mlab.detrend_none,
-              usevlines=True, maxlags=10, **kwargs):
-        r"""
-        Plot the cross correlation between *x* and *y*.
+          usevlines=True, maxlags=10, **kwargs):
+    r"""
+    Plot the cross correlation between *x* and *y*.
 
-        The correlation with lag k is defined as
-        :math:`\sum_n x[n+k] \cdot y^*[n]`, where :math:`y^*` is the complex
-        conjugate of :math:`y`.
+    The correlation with lag k is defined as
+    :math:`\sum_n x[n+k] \cdot y^*[n]`, where :math:`y^*` is the complex
+    conjugate of :math:`y`.
 
-        Parameters
-        ----------
-        x : array-like of length n
+    Parameters
+    ----------
+    x : array-like of length n
 
-        y : array-like of length n
+    y : array-like of length n
 
-        detrend : callable, optional, default: `mlab.detrend_none`
-            *x* and *y* are detrended by the *detrend* callable. This must be a
-            function ``x = detrend(x)`` accepting and returning an
-            `numpy.array`. Default is no normalization.
+    detrend : callable, optional, default: `mlab.detrend_none`
+        *x* and *y* are detrended by the *detrend* callable. This must be a
+        function ``x = detrend(x)`` accepting and returning an
+        `numpy.array`. Default is no normalization.
 
-        normed : bool, optional, default: True
-            If ``True``, input vectors are normalised to unit length.
+    normed : bool, optional, default: True
+        If ``True``, input vectors are normalised to unit length.
 
-        usevlines : bool, optional, default: True
-            Determines the plot style.
+    usevlines : bool, optional, default: True
+        Determines the plot style.
 
-            If ``True``, vertical lines are plotted from 0 to the xcorr value
-            using `Axes.vlines`. Additionally, a horizontal line is plotted
-            at y=0 using `Axes.axhline`.
+        If ``True``, vertical lines are plotted from 0 to the xcorr value
+        using `Axes.vlines`. Additionally, a horizontal line is plotted
+        at y=0 using `Axes.axhline`.
 
-            If ``False``, markers are plotted at the xcorr values using
-            `Axes.plot`.
+        If ``False``, markers are plotted at the xcorr values using
+        `Axes.plot`.
 
-        maxlags : int, optional, default: 10
-            Number of lags to show. If None, will return all ``2 * len(x) - 1``
-            lags.
+    maxlags : int, optional, default: 10
+        Number of lags to show. If None, will return all ``2 * len(x) - 1``
+        lags.
 
-        Returns
-        -------
-        lags : array (length ``2*maxlags+1``)
-            The lag vector.
-        c : array  (length ``2*maxlags+1``)
-            The auto correlation vector.
-        line : `.LineCollection` or `.Line2D`
-            `.Artist` added to the axes of the correlation:
+    Returns
+    -------
+    lags : array (length ``2*maxlags+1``)
+        The lag vector.
+    c : array  (length ``2*maxlags+1``)
+        The auto correlation vector.
+    line : `.LineCollection` or `.Line2D`
+        `.Artist` added to the axes of the correlation:
 
-            - `.LineCollection` if *usevlines* is True.
-            - `.Line2D` if *usevlines* is False.
-        b : `.Line2D` or None
-            Horizontal line at 0 if *usevlines* is True
-            None *usevlines* is False.
+        - `.LineCollection` if *usevlines* is True.
+        - `.Line2D` if *usevlines* is False.
+    b : `.Line2D` or None
+        Horizontal line at 0 if *usevlines* is True
+        None *usevlines* is False.
 
-        Other Parameters
-        ----------------
-        linestyle : `.Line2D` property, optional
-            The linestyle for plotting the data points.
-            Only used if *usevlines* is ``False``.
+    Other Parameters
+    ----------------
+    linestyle : `.Line2D` property, optional
+        The linestyle for plotting the data points.
+        Only used if *usevlines* is ``False``.
 
-        marker : str, optional, default: 'o'
-            The marker for plotting the data points.
-            Only used if *usevlines* is ``False``.
+    marker : str, optional, default: 'o'
+        The marker for plotting the data points.
+        Only used if *usevlines* is ``False``.
 
-        Notes
-        -----
-        The cross correlation is performed with :func:`numpy.correlate` with
-        ``mode = "full"``.
-        """
-        Nx = len(x)
-        if Nx != len(y):
-            raise ValueError('x and y must be equal length')
+    Notes
+    -----
+    The cross correlation is performed with :func:`numpy.correlate` with
+    ``mode = "full"``.
+    """
+    Nx = len(x)
+    if Nx != len(y):
+        raise ValueError('x and y must be equal length')
 
-        x = detrend(np.asarray(x))
-        y = detrend(np.asarray(y))
+    x = detrend(np.asarray(x))
+    y = detrend(np.asarray(y))
 
-        correls = np.correlate(x, y, mode="full")
+    correls = np.correlate(x, y, mode="full")
 
-        if normed:
-            correls /= np.sqrt(np.dot(x, x) * np.dot(y, y))
+    if normed:
+        correls /= np.sqrt(np.dot(x, x) * np.dot(y, y))
 
-        if maxlags is None:
-            maxlags = Nx - 1
+    if maxlags is None:
+        maxlags = Nx - 1
 
-        if maxlags >= Nx or maxlags < 1:
-            raise ValueError('maxlags must be None or strictly '
-                             'positive < %d' % Nx)
+    if maxlags >= Nx or maxlags < 1:
+        raise ValueError('maxlags must be None or strictly '
+                         'positive < %d' % Nx)
 
-        lags = np.arange(-maxlags, maxlags + 1)
-        correls = correls[Nx - 1 - maxlags:Nx + maxlags]
+    lags = np.arange(-maxlags, maxlags + 1)
+    correls = correls[Nx - 1 - maxlags:Nx + maxlags]
 
 #        if usevlines:
 #            a = self.vlines(lags, [0], correls, **kwargs)
@@ -526,7 +532,7 @@ def xcorr(x, y, normed=True, detrend=mlab.detrend_none,
 #            kwargs.setdefault('linestyle', 'None')
 #            a, = self.plot(lags, correls, **kwargs)
 #            b = None
-        return lags, correls
+    return lags, correls
 
 
 def CalcVgeff(Sig, Tchar, VgsExp=None, Regim='hole', CalType='interp'):
@@ -541,7 +547,6 @@ def CalcVgeff(Sig, Tchar, VgsExp=None, Regim='hole', CalType='interp'):
     Ids = Tchar.GetIds(Vgs=vgs[Inds]) * pq.A
     GM = Tchar.GetGM(Vgs=VgsExp) * pq.S
 
-
     IdsExp = Tchar.GetIds(Vgs=VgsExp) * pq.A
     IdsOff = np.mean(Sig)-IdsExp
     IdsBias = np.mean(Sig)
@@ -551,7 +556,7 @@ def CalcVgeff(Sig, Tchar, VgsExp=None, Regim='hole', CalType='interp'):
         if CalType == 'interp':
             fgm = interpolate.interp1d(Ids[:, 0], vgs[Inds])
             st = fgm(np.clip(Sig, np.min(Ids), np.max(Ids)))*pq.V
-        elif CalType=='linear':
+        elif CalType == 'linear':
             st = Sig/GM
         else:
             print('Calibration Not defined')
@@ -576,12 +581,12 @@ def CalcVgeff(Sig, Tchar, VgsExp=None, Regim='hole', CalType='interp'):
                    'GM': GM,
                    }
 
-    CalSig = NeoSignal(st,
-                       units='V',
-                       t_start=Sig.t_start,
-                       sampling_rate=Sig.sampling_rate,
-                       name=str(Sig.name),
-                       file_origin=Sig.file_origin)
+    CalSig = AnalogSignal(st,
+                          units='V',
+                          t_start=Sig.t_start,
+                          sampling_rate=Sig.sampling_rate,
+                          name=str(Sig.name),
+                          file_origin=Sig.file_origin)
 
 #    CalSig.annotate(**annotations)
     CalSig.array_annotate(**annotations)
@@ -601,21 +606,20 @@ def CalcVgeff2(Sig, Tchar, VgsExp, Regim='hole', CalType='interp'):
     IdsBias = np.array((np.nan,))*pq.A
     IdsOff = np.array((np.nan,))*pq.A
     GM = np.nan*pq.S
-    try:    
-        Ids = Tchar.GetIds(Vgs=vgs[Inds]).flatten() 
-        GM = Tchar.GetGM(Vgs=VgsExp).flatten() 
+    try:
+        Ids = Tchar.GetIds(Vgs=vgs[Inds]).flatten()
+        GM = Tchar.GetGM(Vgs=VgsExp).flatten()
 
-
-        IdsExp = Tchar.GetIds(Vgs=VgsExp).flatten() 
+        IdsExp = Tchar.GetIds(Vgs=VgsExp).flatten()
         IdsOff = np.mean(Sig)-IdsExp
         IdsBias = np.mean(Sig)
-    
+
         Calibrated = np.array((True,))
-    
+
         if CalType == 'interp':
             fgm = interpolate.interp1d(Ids, vgs[Inds])
             st = fgm(np.clip(Sig, np.min(Ids), np.max(Ids)))*pq.V
-        elif CalType=='linear':
+        elif CalType == 'linear':
             st = Sig/GM
         else:
             print('Calibration Not defined')
@@ -641,60 +645,55 @@ def CalcVgeff2(Sig, Tchar, VgsExp, Regim='hole', CalType='interp'):
                    'GM': float(GM.magnitude),
                    }
 
-    CalSig = NeoSignal(st,
-                       units='V',
-                       t_start=Sig.t_start,
-                       sampling_rate=Sig.sampling_rate,
-                       name=str(Sig.name),
-                       file_origin=Sig.file_origin)
+    CalSig = AnalogSignal(st,
+                          units='V',
+                          t_start=Sig.t_start,
+                          sampling_rate=Sig.sampling_rate,
+                          name=str(Sig.name),
+                          file_origin=Sig.file_origin)
 
-    CalSig.annotate(**annotations)
-    # CalSig.array_aºnnotate(**annotations)
+    CalSig.array_annotate(**annotations)
+    # CalSig.array_annotate(**annotations)
 
     return CalSig
 
-def CalcVgeffNoInterp(Sig, Tchar, VgsExp=None, Regim='hole'):    
-    gm=Tchar.GetGM(Vgs=VgsExp)
 
+def CalcVgeffNoInterp(Sig, Tchar, VgsExp=None, Regim='hole'):
+    gm = Tchar.GetGM(Vgs=VgsExp)
 
     Calibrated = np.array((True,))
     try:
-        st=Sig.magnitude/gm
+        st = Sig.magnitude/gm
     except:
         print(Sig.name, "Calibration error:", sys.exc_info()[0])
         st = np.zeros(Sig.shape)
         Calibrated = np.array((False,))
-        
-        
+
     print(str(Sig.name), '-> ', 'GM', gm, 'Vgs', np.mean(st), Tchar.IsOK)
     annotations = {'Calibrated': Calibrated,
-                   'Working':Calibrated,
+                   'Working': Calibrated,
                    # 'IdsOff': IdsOff.flatten(),
                    'VgsCal': np.array((np.mean(st), )),
                    'IsOK': np.array((Tchar.IsOK, )),
-                   'Iname': np.array((Sig.name, )),                   
+                   'Iname': np.array((Sig.name, )),
                    }
-    
-    CalSig = NeoSignal(st*pq.V,
-                       units='V',
-                       t_start=Sig.t_start,
-                       sampling_rate=Sig.sampling_rate,
-                       name=str(Sig.name),
-                       file_origin=Sig.file_origin)
 
-#    CalSig.annotate(**annotations)    
+    CalSig = AnalogSignal(st*pq.V,
+                          units='V',
+                          t_start=Sig.t_start,
+                          sampling_rate=Sig.sampling_rate,
+                          name=str(Sig.name),
+                          file_origin=Sig.file_origin)
+
+#    CalSig.annotate(**annotations)
     CalSig.array_annotate(**annotations)
-        
+
     return CalSig
 
 
-
 def ApplyProcessChain(sig, ProcessChain):
-    sl = sig.copy()  
+    sl = sig.copy()
     for Proc in ProcessChain:
         sl = Proc['function'](sl, **Proc['args'])
-    
+
     return sl
-
-
-
